import 'context.dart';
import 'filters.dart';
import 'loaders.dart';
import 'nodes.dart';
import 'parser.dart';
import 'tests.dart';

class Template {
  const Template._({this.path, this.root});
  Template(String source, [String path])
      : path = path,
        root = Parser(source, uri: path).parse();

  final String path;
  final Node root;

  String render([Map<String, dynamic> context]) =>
      root.render(Context(data: context));

  String renderContext(Context context) => root.render(context);

  @override
  bool operator ==(dynamic other) =>
      other is Template && other.path == path && other.root == root;

  @override
  int get hashCode => path.hashCode ^ root.hashCode ^ 290418677;

  @override
  String toString() => 'Template($root${path != null ? ', ' + path : ''})';
}

class Env {
  static Env instance;

  Env({
    this.debug = true,
    this.ext = const <String>['html'],
    Loader loader,
    Map<String, Function> filters = const <String, Function>{},
    Map<String, Function> tests = const <String, Function>{},
    this.stmtOpen = '{%',
    this.stmtClose = '%}',
    this.varOpen = '{{',
    this.varClose = '}}',
    this.commentOpen = '{#',
    this.commentClose = '#}',
  })  : loader = loader ?? FileSystemLoader('lib/src/templates'),
        templates = <String, Template>{},
        filters = Map<String, Function>.of(FILTERS)..addAll(filters),
        tests = Map<String, Function>.of(TESTS)..addAll(tests) {
    Env.instance = this;
    this.loader.load(this);
  }

  final bool debug;
  final List<String> ext;
  final Loader loader;
  final Map<String, Template> templates;
  final Map<String, Function> filters;
  final Map<String, Function> tests;
  final String stmtOpen;
  final String stmtClose;
  final String varOpen;
  final String varClose;
  final String commentOpen;
  final String commentClose;

  Template getTemplate(String path) {
    if (!templates.containsKey(path)) throw Exception('Template not found: $path');
    return templates[path];
  }

  Function getFilter(String name) {
    if (!filters.containsKey(name)) throw Exception('Filter not found: $name');
    return filters[name];
  }

  Function getTest(String name) {
    if (!tests.containsKey(name)) throw Exception('Test not found: $name');
    return tests[name];
  }

  @override
  String toString() => 'Env($debug, $ext, $loader)';
}
